@@ -19,7 +19,6 @@
 
 package org.apache.cxf.transport.http;
 
-import java.io.IOException;
 import java.io.OutputStream;
 import java.net.MalformedURLException;
 import java.net.URL;
@@ -34,22 +33,24 @@ import java.util.logging.Logger;
 
 import javax.wsdl.Definition;
 import javax.wsdl.Import;
+import javax.wsdl.Port;
+import javax.wsdl.Service;
 import javax.wsdl.Types;
-import javax.wsdl.WSDLException;
 import javax.wsdl.extensions.ExtensibilityElement;
 import javax.wsdl.extensions.schema.Schema;
 import javax.wsdl.extensions.schema.SchemaImport;
 import javax.wsdl.extensions.schema.SchemaReference;
+import javax.wsdl.extensions.soap.SOAPAddress;
+import javax.wsdl.extensions.soap12.SOAP12Address;
 import javax.wsdl.xml.WSDLWriter;
-import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.namespace.QName;
+import javax.xml.stream.XMLStreamWriter;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
 
 import org.xml.sax.InputSource;
-import org.xml.sax.SAXException;
-
 
 import org.apache.cxf.Bus;
 import org.apache.cxf.catalog.OASISCatalogManager;
@@ -58,6 +59,7 @@ import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.helpers.XMLUtils;
 import org.apache.cxf.service.model.EndpointInfo;
+import org.apache.cxf.staxutils.StaxUtils;
 import org.apache.cxf.transports.http.StemMatchingQueryHandler;
 import org.apache.cxf.wsdl.WSDLManager;
 import org.apache.cxf.wsdl11.ResourceManagerWSDLLocator;
@@ -148,11 +150,21 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                                                        base);
                     def = mp.get(wsdl2);
                 }
-    
-                WSDLWriter wsdlWriter = bus.getExtension(WSDLManager.class)
-                    .getWSDLFactory().newWSDLWriter();
-                def.setExtensionRegistry(bus.getExtension(WSDLManager.class).getExtenstionRegistry());
-                doc = wsdlWriter.getDocument(def);
+                
+                synchronized (def) {
+                    //writing a def is not threadsafe.  Sync on it to make sure
+                    //we don't get any ConcurrentModificationExceptions
+                    if (endpointInfo.getProperty("publishedEndpointUrl") != null) {
+                        String publishingUrl = 
+                            String.valueOf(endpointInfo.getProperty("publishedEndpointUrl"));
+                        updatePublishedEndpointUrl(publishingUrl, def, endpointInfo.getName());
+                    }
+        
+                    WSDLWriter wsdlWriter = bus.getExtension(WSDLManager.class)
+                        .getWSDLFactory().newWSDLWriter();
+                    def.setExtensionRegistry(bus.getExtension(WSDLManager.class).getExtensionRegistry());
+                    doc = wsdlWriter.getDocument(def);
+                }
             } else {
                 SchemaReference si = smp.get(xsd);
                 if (si == null) {
@@ -206,27 +218,22 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                     el.setAttribute("location", base + "?wsdl=" + sl);
                 }
             }
-            
-            XMLUtils.writeTo(doc, os);
-        } catch (WSDLException wex) {
+            doc.setXmlStandalone(true);
+            String enc = doc.getXmlEncoding();
+            if (enc == null) {
+                enc = "utf-8";
+            }
+
+            XMLStreamWriter writer = StaxUtils.createXMLStreamWriter(os,
+                                                                     enc);
+            StaxUtils.writeNode(doc, writer, true);
+            writer.flush();
+        } catch (Exception wex) {
             throw new WSDLQueryException(new Message("COULD_NOT_PROVIDE_WSDL",
                                                      LOG,
                                                      baseUri), wex);
-        } catch (SAXException e) {
-            throw new WSDLQueryException(new Message("COULD_NOT_PROVIDE_WSDL",
-                                                     LOG,
-                                                     baseUri), e);
-        } catch (IOException e) {
-            throw new WSDLQueryException(new Message("COULD_NOT_PROVIDE_WSDL",
-                                                     LOG,
-                                                     baseUri), e);
-        } catch (ParserConfigurationException e) {
-            throw new WSDLQueryException(new Message("COULD_NOT_PROVIDE_WSDL",
-                                                     LOG,
-                                                     baseUri), e);
         }
     }
-    
 
     static String resolveWithCatalogs(OASISCatalogManager catalogs, String start, String base) {
         String resolvedSchemaLocation = null;
@@ -286,6 +293,38 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                 }
             }
         }
+    }    
+
+    protected void updatePublishedEndpointUrl(String publishingUrl, Definition def, QName name) {
+        Collection<Service> services = CastUtils.cast(def.getAllServices().values());
+        for (Service service : services) {
+            Collection<Port> ports = CastUtils.cast(service.getPorts().values());
+            if (ports.isEmpty()) {
+                continue;
+            }
+            
+            if (name == null) {
+                setSoapAddressLocationOn(ports.iterator().next(), publishingUrl);
+                break; // only update the first port since we don't target any specific port
+            } else {
+                for (Port port : ports) {
+                    if (name.getLocalPart().equals(port.getName())) {
+                        setSoapAddressLocationOn(port, publishingUrl);
+                    }
+                }
+            }
+        }
+    }
+    
+    private void setSoapAddressLocationOn(Port port, String url) {
+        List extensions = port.getExtensibilityElements();
+        for (Object extension : extensions) {
+            if (extension instanceof SOAP12Address) {
+                ((SOAP12Address)extension).setLocationURI(url);
+            } else if (extension instanceof SOAPAddress) {
+                ((SOAPAddress)extension).setLocationURI(url);
+            }
+        }
     }
     
     protected void updateSchemaImports(Schema schema,
